% Deutsche kontextfreie Grammatik
% Autor: Syntaxkurs 2013/2014
% Version 5

%% Meilensteine
%%
%% - Version 1: Kongruenz in Kasus
%% - Version 2: Verwendung von Wrappern und Tests.
%% - Version 3: Verwendung von Pronomen.
%%- Version 4: Kongruenz in Kasus, Numerus und Genus. Logische Trennung von Formmerkmalen und Artmerkmalen.
%% - Version 5: Rektion von Verben und Präpositionen in Kasus.
%% - Version 6: Verallgemeinerung des Lexikons.
%% - Version 7: Semantische Kongruenz und Rektion.
%% - Version 8: Verallgemeinerung der Grammatik zu generischen Regeln.

:- consult(dkfg_tools).
:- consult(dkfg_tests_v5).

s -->
    np(nom),
    vp.

vp -->
    v.
vp -->
    v_1(Kasus),
    np_s(Kasus),
    pp.
vp -->
    v_2(Kasus),
    np(Kasus).
vp -->
    v_1(Kasus),
    np_s(Kasus).

np(Kasus) -->
    np_s(Kasus).
np(Kasus) -->
    np_ss(Kasus),
    pp.

np_s(Kasus) -->
    np_ss(Kasus).

np_s(Kasus) -->
    pron([Kasus, _], [_]).
np_ss(Kasus) -->
    en([Kasus, _], [_]).

np_ss(Kasus) -->
    det([Kasus, Numerus], [Genus]),
    n([Kasus, Numerus], [Genus]).

np_ss(Kasus) -->
    det([Kasus, Numerus], [Genus]),
    a([Kasus, Numerus], [Genus]),
    n([Kasus, Numerus], [Genus]).

pp -->
    p(Kasus),
    np_s(Kasus).

en([nom,sg], [m]) --> ['Sigurd'].
en([nom,sg], [f]) --> ['Brunhild'].

a([nom,sg], [m]) --> ['alte'].
a([dat,sg], [m]) --> ['alten'].
a([akk,sg], [m]) --> ['alten'].
a([nom,sg], [f]) --> ['alte'].
a([dat,sg], [f]) --> ['alte'].
a([akk,sg], [f]) --> ['alte'].

n([nom,sg], [m]) --> ['Mann'].
n([dat,sg], [m]) --> ['Mann'].
n([akk,sg], [m]) --> ['Mann'].
n([nom,sg], [m]) --> ['Met'].
n([dat,sg], [m]) --> ['Met'].
n([akk,sg], [m]) --> ['Met'].
n([nom,sg], [m]) --> ['Becher'].
n([dat,sg], [m]) --> ['Becher'].
n([akk,sg], [m]) --> ['Becher'].
n([nom,sg], [f]) --> ['Frau'].
n([dat,sg], [f]) --> ['Frau'].
n([akk,sg], [f]) --> ['Frau'].
n([nom,sg], [n]) --> ['Horn'].
n([dat,sg], [n]) --> ['Horn'].
n([akk,sg], [n]) --> ['Horn'].

pron([nom,sg], [m]) --> ['er'].
pron([dat,sg], [m]) --> ['ihm'].
pron([akk,sg], [m]) --> ['ihn'].
pron([nom,sg], [f]) --> ['sie'].
pron([dat,sg], [f]) --> ['sie'].
pron([akk,sg], [f]) --> ['sie'].

det([nom,sg], [m]) --> ['der'].
det([dat,sg], [m]) --> ['dem'].
det([akk,sg], [m]) --> ['den'].
det([nom,sg], [f]) --> ['die'].
det([dat,sg], [f]) --> ['der'].
det([akk,sg], [f]) --> ['die'].
det([nom,sg], [n]) --> ['das'].
det([dat,sg], [n]) --> ['dem'].
det([akk,sg], [n]) --> ['das'].

v_1(akk) --> ['trinkt'].
v_2(akk) --> ['mag'].
v --> ['schläft'].

p(dat) --> ['aus'].

%% det(Form, Art)      --> [Lexem], { lex(Lexem, det,  Form, Art) }.
%% n(Form, Art)        --> [Lexem], { lex(Lexem, n,    Form, Art) }.
%% a(Form, Art)        --> [Lexem], { lex(Lexem, a,    Form, Art) }.
%% en(Form, Art)       --> [Lexem], { lex(Lexem, en,   Form, Art) }.
%% pron(Form, Art)     --> [Lexem], { lex(Lexem, pron, Form, Art) }.
%% v(Form, Art)        --> [Lexem], { lex(Lexem, v,    Form, Art) }.
%% p(Form, Art)        --> [Lexem], { lex(Lexem, p,    Form, Art) }.
